import 'package:drift/drift.dart';
import 'package:drift_flutter/drift_flutter.dart';

part 'database.g.dart';

class Sessions extends Table {
  IntColumn get id => integer().autoIncrement()();
  TextColumn get title => text().withLength(min: 3, max: 32)();
  TextColumn get content => text().named('body')();
  DateTimeColumn get date => dateTime().nullable()();
  DateTimeColumn get createdAt => dateTime().nullable()();
}

class SessionActivities extends Table {
  IntColumn get id => integer().autoIncrement()();
  IntColumn get sessionId => integer().references(Sessions, #id)();
  IntColumn get activityId => integer().references(Activities, #id)();
  TextColumn get results => text()();
  TextColumn get achievements => text()();
  DateTimeColumn get createdAt => dateTime().nullable()();
}

enum Categories {
   fundamentals,
   conditioning,
   advanced,
   custom,
   pro
}

class Activities extends Table {
  IntColumn get id => integer().autoIncrement()();
  TextColumn get title => text().withLength(min: 3, max: 32)();
  TextColumn get type => text().withLength(min: 3, max: 32)();
  TextColumn get description => text().named('body')();
  TextColumn get categories => textEnum<Categories>()();
  DateTimeColumn get createdAt => dateTime().nullable()();
}

class ActivityActions extends Table {
  IntColumn get id => integer().autoIncrement()();
  IntColumn get activityId => integer().references(Activities, #id)();
  IntColumn get actionId => integer().references(Actions, #id)();
  DateTimeColumn get createdAt => dateTime().nullable()();
}

class Actions extends Table {
  IntColumn get id => integer().autoIncrement()();
  TextColumn get title => text().withLength(min: 3, max: 32)();
  TextColumn get description => text().named('body')();
  TextColumn get set => text()();
  DateTimeColumn get createdAt => dateTime().nullable()();
}

class ObjectMediaItems extends Table {
  IntColumn get id => integer().autoIncrement()();
  IntColumn get objectId => integer().references(Actions, #id)();
  IntColumn get mediaId => integer().references(MediaItems, #id)();
  DateTimeColumn get createdAt => dateTime().nullable()();
}

class MediaItems extends Table {
  IntColumn get id => integer().autoIncrement()();
  TextColumn get title => text().withLength(min: 3, max: 32)();
  TextColumn get description => text().named('body')();
  TextColumn get reference => text().withLength(min: 3, max: 32)();
  TextColumn get type => text().withLength(min: 3, max: 32)();
  DateTimeColumn get createdAt => dateTime().nullable()();
}

@DriftDatabase(tables: [
  Sessions,
  SessionActivities,
  Activities,
  ActivityActions,
  Actions,
  ObjectMediaItems,
  MediaItems
])

class AppDatabase extends _$AppDatabase {
  // After generating code, this class needs to define a `schemaVersion` getter
  // and a constructor telling drift where the database should be stored.
  // These are described in the getting started guide: https://drift.simonbinder.eu/setup/
  AppDatabase() : super(_openConnection());

  @override
  int get schemaVersion => 1;

  static QueryExecutor _openConnection() {
    // `driftDatabase` from `package:drift_flutter` stores the database in
    // `getApplicationDocumentsDirectory()`.
    return driftDatabase(name: 'sendtrain');
  }
}
